<?php

namespace WpBeacon\Actions;

use WP_Application_Passwords;
use WP_User;
use WpBeacon\Providers\MonitoringUserProvider;
use WpBeacon\Support\AdminNotice;

class MonitoringUserAction
{
    const PASSWORD_LENGTH = 42;
    const APP_PASSWORD_NAME = 'Cyrano Monitoring Application Password';
    const ERROR_CREATING_USER = 'Error creating monitoring user.';
    const ERROR_CREATING_PASSWORD = 'Error creating application password.';

    public static function updateOrCreate(): void
    {
        $name = MonitoringUserProvider::USERNAME;
        $email = MonitoringUserProvider::EMAIL;
        $capability = MonitoringUserProvider::CAPABILITY;

        $userId = email_exists($email);
        if (!$userId) {
            self::createUser($name, $email, $capability);
            return;
        }

        self::setSystemUserCapabilities($userId, $capability);
    }

    private static function createUser($name, $email, $capability): void
    {
        $password = wp_generate_password(self::PASSWORD_LENGTH);
        $userId = wp_create_user($name, $password, $email);

        if (is_wp_error($userId)) {
            AdminNotice::error(self::ERROR_CREATING_USER);
            return;
        }

        self::setSystemUserCapabilities($userId, $capability);
        self::createApplicationPassword($userId, $name);
    }

    private static function createApplicationPassword($userId, $name): void
    {
        $appPasswordData = [
            'name' => self::APP_PASSWORD_NAME
        ];
        $appPassword = WP_Application_Passwords::create_new_application_password($userId, $appPasswordData);

        if (is_wp_error($appPassword)) {
            AdminNotice::error(self::ERROR_CREATING_PASSWORD);
            return;
        }

        $notice = "
        <h2>Zugangsdaten fürs Cyrano WordPress Monitoring</h2>
        Benutzername: <code id='username'>$name</code>
        <button class='copy-button' data-clipboard-target='#username'>
            <svg xmlns='http://www.w3.org/2000/svg' width='24' height='24' viewBox='0 0 24 24' fill='none' stroke='currentColor' stroke-width='1' stroke-linecap='round' stroke-linejoin='round' class='lucide lucide-clipboard-copy'>
                <rect width='8' height='4' x='8' y='2' rx='1' ry='1'/>
                <path d='M8 4H6a2 2 0 0 0-2 2v14a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2v-2'/>
                <path d='M16 4h2a2 2 0 0 1 2 2v4'/>
                <path d='M21 14H11'/>
                <path d='m15 10-4 4 4 4'/>
            </svg>
        </button><br>
        Passwort: <code id='password'>{$appPassword[0]}</code>
        <button class='copy-button' data-clipboard-target='#password'>
            <svg xmlns='http://www.w3.org/2000/svg' width='24' height='24' viewBox='0 0 24 24' fill='none' stroke='currentColor' stroke-width='1' stroke-linecap='round' stroke-linejoin='round' class='lucide lucide-clipboard-copy'>
                <rect width='8' height='4' x='8' y='2' rx='1' ry='1'/>
                <path d='M8 4H6a2 2 0 0 0-2 2v14a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2v-2'/>
                <path d='M16 4h2a2 2 0 0 1 2 2v4'/>
                <path d='M21 14H11'/>
                <path d='m15 10-4 4 4 4'/>
            </svg>
        </button>
        <script>
            document.addEventListener('DOMContentLoaded', function() {
                var buttons = document.querySelectorAll('.copy-button');
                buttons.forEach(function(button) {
                    button.addEventListener('click', function(event) {
                        var target = document.querySelector(button.getAttribute('data-clipboard-target'));
                        var range = document.createRange();
                        range.selectNode(target);
                        window.getSelection().removeAllRanges();
                        window.getSelection().addRange(range);
                        try {
                            document.execCommand('copy');
                            window.getSelection().removeAllRanges();

                            // Change button SVG to checkmark and color to green
                            button.innerHTML = `
                                <svg xmlns='http://www.w3.org/2000/svg' width='24' height='24' viewBox='0 0 24 24' fill='none' stroke='currentColor' stroke-width='2' stroke-linecap='round' stroke-linejoin='round' class='lucide lucide-check' style='color:green'>
                                    <path d='M20 6 9 17l-5-5'/>
                                </svg>
                            `;
                        } catch (err) {
                            console.error('Failed to copy!', err);
                        }
                    });
                });
            });
        </script>
    ";

        AdminNotice::success($notice);
    }

    public static function setSystemUserCapabilities(int $userId, string $capability): void
    {
        $user = new WP_User($userId);
        $user->description = 'System user: ' . MonitoringUserProvider::USERNAME;
        $user->set_role('');

        $user->add_cap($capability);
        foreach ($user->allcaps as $cap => $value) {
            if ($cap != $capability) {
                $user->remove_cap($cap);
            }
        }
    }
}
